import React from 'react';
import { lowlight } from 'lowlight';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var _excluded = ["children"];

var cloneHastNode = function cloneHastNode(_ref) {
  var children = _ref.children,
      type = _ref.type,
      value = _ref.value,
      properties = _ref.properties,
      tagName = _ref.tagName;
  return {
    type: type,
    value: value,
    tagName: tagName,
    children: children ? children.map(cloneHastNode) : null,
    properties: _extends({}, properties)
  };
};

var splitNewLine = function splitNewLine(hastNode) {
  if (hastNode.type === 'text') {
    if (hastNode.value.indexOf('\n') === -1) {
      return [hastNode];
    } else {
      var texts = hastNode.value.split('\n');
      return texts.map(function (text, index) {
        return {
          type: 'text',
          value: text
        };
      });
    }
  } else if (hastNode.children && hastNode.children.length) {
    var children = hastNode.children,
        newParent = _objectWithoutPropertiesLoose(hastNode, _excluded);

    newParent.children = [];
    var out = [cloneHastNode(newParent)];
    children.forEach(function (childHastNode) {
      var hastLineNodes = splitNewLine(childHastNode);
      hastLineNodes.forEach(function (hastLineNode, index) {
        out[out.length - 1].children.push(hastLineNode);

        if (index + 1 < hastLineNode.length) {
          out.push(cloneHastNode(newParent));
        }
      });
    });
    return out;
  } else {
    return [hastNode];
  }
};

var newLine = function newLine(num) {
  return [{
    type: 'element',
    tagName: 'span',
    properties: {
      style: {
        gridColumn: 1
      }
    },
    children: [{
      type: 'text',
      value: num
    }]
  }, {
    type: 'element',
    tagName: 'span',
    properties: {
      style: {
        gridColumn: 2,
        whiteSpace: 'pre-wrap'
      }
    },
    children: []
  }];
};

var groupLines = function groupLines(hastNodes, firstLineNumber) {
  var hastNodesOutput = newLine(firstLineNumber);
  hastNodes.forEach(function (hastNode) {
    var hastLineNodes = splitNewLine(hastNode);
    hastLineNodes.forEach(function (hastLineNode, index) {
      hastNodesOutput[hastNodesOutput.length - 1].children.push(hastLineNode);

      if (index + 1 < hastLineNodes.length) {
        hastNodesOutput = [].concat(hastNodesOutput, newLine(hastNodesOutput.length / 2 + firstLineNumber));
      }
    });
  });
  return hastNodesOutput;
};

function mapChild(child, i, depth) {
  if (child.tagName) {
    return React.createElement(child.tagName, _extends({
      key: "lo-" + depth + "-" + i
    }, child.properties), child.children && child.children.map(toReactComponents(depth + 1)));
  }

  return child.value;
}

function toReactComponents(depth) {
  return function mapChildrenWithDepth(child, i) {
    return mapChild(child, i, depth);
  };
}

var Syntaxx = function Syntaxx(_ref) {
  var language = _ref.language,
      value = _ref.value,
      children = _ref.children,
      firstLineNumber = _ref.firstLineNumber;

  var _React$useState = React.useState([]),
      reactComponents = _React$useState[0],
      setReactComponents = _React$useState[1];

  var code = children || value;
  React.useEffect(function () {
    var cleanedCode = code.replace(/^\n/, '');
    var lowlightHighlight = lowlight.highlight(language, cleanedCode);
    var hast = lowlightHighlight.children;
    var groupedHast = groupLines(hast || [], firstLineNumber);
    setReactComponents(groupedHast.map(toReactComponents(0)));
  }, [language, code, firstLineNumber]);
  return /*#__PURE__*/React.createElement("pre", null, /*#__PURE__*/React.createElement("code", null, /*#__PURE__*/React.createElement("span", {
    className: "syntaxx-grid",
    style: {
      display: 'grid',
      gridTemplateColumns: 'auto 1fr'
    }
  }, reactComponents)));
};

Syntaxx.defaultProps = {
  firstLineNumber: 1
};

export default Syntaxx;
//# sourceMappingURL=index.modern.js.map
